using System;
using RenovationWorkContracts.BindingModels;
using RenovationWorkContracts.ViewModels;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using RenovationWorkDataModels.Models;
using RenovationWorkDatabaseImplement.Models;
using System.Collections.Generic;

namespace RenovationWorkDatabaseImplement
{
    public class Component : IComponentModel
    {
        public int Id { get; private set; }
        [Required]
        public string ComponentName { get; private set; } = string.Empty;
        [Required]
        public double Cost { get; set; }
        [ForeignKey("ComponentId")]
        public virtual List<RepairComponent> RepairComponents { get; set; } = new();

        public static Component? Create(ComponentBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
            return new Component()
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost
            };
        }

        public static Component Create(ComponentViewModel model)
        {
            return new Component
            {
                Id = model.Id,
                ComponentName = model.ComponentName,
                Cost = model.Cost
            };
        }

        public void Update(ComponentBindingModel model)
        {
            if (model == null)
            {
                return;
            }
            ComponentName = model.ComponentName;
            Cost = model.Cost;
        }

        public ComponentViewModel GetViewModel => new()
        {
            Id = Id,
            ComponentName = ComponentName,
            Cost = Cost
        };
    }
}

